--[[----------------------------------------------------------------------
      DataGatherer Module - Part of VanasKoS
Handles all external Player Data from Chat and Target Changes/Mouseovers
------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("VanasKoSDataGatherer");

VanasKoSDataGatherer = VanasKoS:NewModule("DataGatherer");

local zone = "";

function VanasKoSDataGatherer:OnInitialize()
	self.db = VanasKoS.db;
	
	VanasKoS:RegisterList(nil, "PLAYERDATA", nil, self);
	VanasKoS:RegisterList(nil, "GUILDDATA", nil, self);
end

function VanasKoSDataGatherer:OnEnable()
	-- Mouseover, Targetchanges
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");

	-- Chat Messages
	self:RegisterEvent("CHAT_MSG_HOSTILEPLAYER_BUFF", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_HOSTILEPLAYER_DAMAGE", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS", "ChatCombatMessageHandlerEnemy");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", "ChatCombatMessageHandlerEnemy");

	-- Chat Messages Friendly - to gather messages & friendly targets
	self:RegisterEvent("CHAT_MSG_FRIENDLYPLAYER_BUFF", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_FRIENDLYPLAYER_DAMAGE", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", "ChatCombatMessageHandlerFriendly");
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", "ChatCombatMessageHandlerFriendly");

	-- say, emotes
	self:RegisterEvent("CHAT_MSG_SAY", "ChatMessageAndEmoteHandler");
	self:RegisterEvent("CHAT_MSG_EMOTE", "ChatMessageAndEmoteHandler");
	self:RegisterEvent("CHAT_MSG_TEXT_EMOTE", "ChatMessageAndEmoteHandler");
	
	-- on zonechange update zone
	self:RegisterEvent("ZONE_CHANGED", "UpdateZone");
	self:RegisterEvent("ZONE_CHANGED_INDOORS", "UpdateZone");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "UpdateZone");
	
	self:RegisterEvent("VanasKoS_Player_Data_Gathered", "Data_Gathered");
	self:RegisterEvent("VanasKoS_Player_Detected", "Player_Detected");

	self:UpdateZone();
end

function VanasKoSDataGatherer:OnDisable()
	self:UnregisterAllEvents();
end

-- if a player was detected nearby through chat or mouseover, update lastseen
function VanasKoSDataGatherer:Player_Detected(name, guild, faction)
	local reason, list = VanasKoS:IsOnList(nil, name)
	
	if(list ~= nil) then
		-- if the player is on a relevant list, update last seen
		self:UpdateLastSeen(name);
	end
end

function VanasKoSDataGatherer:IsOnList(listname, name, guild)
	local listVar = VanasKoS:GetList(listname);
	if(listVar and listVar[name]) then
		local creator = nil;
		if(listVar[name].creator) then
			creator = listVar[name].creator;
		end
		return listVar[name].reason, creator;
	else
		return nil;
	end
end


function VanasKoSDataGatherer:GetList(list)
	if(list == "PLAYERDATA") then
		return self.db.realm.data.players;
	elseif(list == "GUILDDATA") then
		return self.db.realm.data.guilds;
	else
		return nil;
	end
end

function VanasKoSDataGatherer:AddEntry(list, name, data)
	return true;
end

function VanasKoSDataGatherer:RemoveEntry(listname, name)
	local list = VanasKoS:GetList(listname);
	if(list and list[name]) then
		list[name] = nil;
		self:TriggerEvent("VanasKoS_List_Entry_Removed", listname, name);
	end
end

function VanasKoSDataGatherer:UpdateLastSeen(name)
	local list = VanasKoS:GetList("PLAYERDATA");
	if(list[string.lower(name)] ~= nil) then
		list[string.lower(name)].lastseen = time();
	end
end

function VanasKoSDataGatherer:Data_Gathered(list, name, data)
	-- self:PrintLiteral(list, name, guild, level, race, class, gender);
	if(name == nil or list == nil) then
		return;
	end
	
	if(list ~= nil and name ~= nil) then
		local listVar = VanasKoS:GetList("PLAYERDATA");
		
		if(not listVar[string.lower(name)]) then
			listVar[string.lower(name)] = { };
		end
		self:UpdateLastSeen(name);
		
		listVar[string.lower(name)].displayname = name;
		listVar[string.lower(name)].guild = data['guild'];
		listVar[string.lower(name)].level = data['level'];
		listVar[string.lower(name)].race = data['race'];
		listVar[string.lower(name)].class = data['class'];
		listVar[string.lower(name)].gender = data['gender'];
		listVar[string.lower(name)].zone = data['zone'];
		
		if(guild ~= nil and VanasKoS:IsKoSGuild(guild)) then
			local guildData = VanasKoS:GetList("GUILDDATA");
			if(not guildData[string.lower(guild)]) then
				guildData[string.lower(guild)] = { };
			end
			guildData[string.lower(guild)].displayname = guild;
		end
		
		VanasKoSGUI:Update();
	end
end

local continent = -1;
local zoneID = -1;

function VanasKoSDataGatherer:UpdateZone()
	continent = GetCurrentMapContinent();
	local zoneNames = { GetMapZones(continent) };
	zoneID = GetCurrentMapZone();
	zone = zoneNames[zoneID];
	
	if(continent == -1) then
		zone = GetZoneText();
	end

	if(zone == nil) then
		return;
	end
	
	self:TriggerEvent("VanasKoS_Zone_Changed", zone, continent, zoneID);
end

function VanasKoSDataGatherer:IsInShattrath()
	if(continent == 3 and zoneID == 6) then
		return true;
	else
		return false;
	end
end

function VanasKoSDataGatherer:PLAYER_TARGET_CHANGED()
	local name = nil;
	local guild = nil;
	local faction = nil;
	if(UnitIsPlayer("target")) then
		name = UnitName("target");
		guild = GetGuildInfo("target");

		if(name == nil) then
			return;
		end
		
		local data = self:Get_Player_Data("target");

		local faction = nil;
		if(VanasKoS:IsKoSPlayer(name) or VanasKoS:IsKoSGuild(guild)) then
			faction = "kos";
		elseif(UnitExists("target") and UnitIsEnemy("player", "target")) then
			faction = "enemy";
		else
			faction = "friendly";
		end
		
		if(name ~= nil) then
			self:TriggerEvent("VanasKoS_Player_Detected", name, guild, faction, data);
		end
		
		self:TriggerEvent("VanasKoS_Player_Target_Changed", name, guild, faction);
	else
		self:TriggerEvent("VanasKoS_Mob_Target_Changed", nil, nil, nil);
	end
end

function VanasKoSDataGatherer:Get_Player_Data(unit)
	local data = nil;
	if(UnitIsPlayer(unit)) then
		local name = UnitName(unit);
		local guild = GetGuildInfo(unit);
		local level = UnitLevel(unit);
		local race = UnitRace(unit);
		local class = UnitClass(unit);
		local gender = UnitSex(unit);
		
		if(name == nil) then
			return;
		end
		local reason, list = VanasKoS:IsOnList(nil, name, guild);
		data = { ['guild'] = guild, 
					['level'] = level, 
					['race'] = race, 
					['class'] = class, 
					['gender'] = gender, 
					['zone'] = zone };
		self:TriggerEvent("VanasKoS_Player_Data_Gathered", list, name, data);
	end
	return data;
end

function VanasKoSDataGatherer:UPDATE_MOUSEOVER_UNIT()
	if(UnitIsPlayer("mouseover")) then
		local name = UnitName("mouseover");
		local guild = GetGuildInfo("mouseover");

		local data = self:Get_Player_Data("mouseover");
		
		local faction = nil;
		if(VanasKoS:IsKoSPlayer(name) or VanasKoS:IsKoSGuild(guild)) then
			faction = "kos";
		elseif(UnitExists("mouseover") and UnitIsEnemy("player", "mouseover")) then
			faction = "enemy";
		else
			faction = "friendly";
		end
		
		if(name ~= nil) then
			self:TriggerEvent("VanasKoS_Player_Detected", name, guild, faction, data);
		end

		self:TriggerEvent("VanasKoS_Player_MouseOver", name, guild);
	end
end

function VanasKoSDataGatherer:ChatCombatMessageHandlerFriendly()
	self:ChatCombatMessageHandler("friendly");
end

function VanasKoSDataGatherer:ChatCombatMessageHandlerEnemy()
	self:ChatCombatMessageHandler("enemy");
end

local patternMap = VanasKoS_Parser_MessageMap;

function VanasKoSDataGatherer:GetPlayerOfChatCombatMessage(message)
	for index, value in pairs(patternMap) do
		--self:Print("results " .. message .. " " .. value.pattern);
		local results = { message:find(value.pattern) };
		if(#results > 0) then
			local name = nil;
			--self:PrintLiteral(message, results, value.map);
			if(value.map['source'] ~= nil) then
				name = results[value.map['source'] + 2]:trim();
			elseif(value.map['target'] ~= nil) then
				name = results[value.map['target'] + 2]:trim();
			end
			
			if(name and not name:find(" ")) then
				return name;
			else
				if(VANASKOS.DEBUG == 1) then
					self:Print("[DEBUG]: Parsing: " .. message .. " yielded name with spaces -- name:" .. name .. " -- pattern:" .. value.pattern );
				end
			end
		end
	end
	if(VANASKOS.DEBUG == 1) then
		self:Print("[DEBUG]: Message: " .. message .. " couldn't be parsed");
	end
	return nil;
end

function VanasKoSDataGatherer:ChatMessageAndEmoteHandler()
	if(VanasKoS:IsKoSPlayer(arg2)) then
		self:TriggerEvent("VanasKoS_Player_Detected", arg2, nil, "kos");
	end
end

function VanasKoSDataGatherer:ChatCombatMessageHandler(faction)
  if(arg1) then
  	local name = self:GetPlayerOfChatCombatMessage(arg1);
  	if(name == nil) then
  		return nil;
  	end
	
	name = string.trim(name);
	
    if(VanasKoS:IsKoSPlayer(name)) then
		self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "kos");
    end
    if(faction == "enemy") then
		-- workaround - blizzard flags party members as enemy
		if(GetNumPartyMembers() ~= 0) then
			for i = 1, 4 do
				local partyname = UnitName("party" .. i);
				if(partyname ~= nil and partyname == name) then
					self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "friendly");
					return nil;
				end
			end
		end
		
		self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "enemy");
    end
    if(faction == "friendly") then
		self:TriggerEvent("VanasKoS_Player_Detected", name, nil, "friendly");
    end
  end
end 

